@@ -324,7 +324,7 @@ impl<T: Clone> MergeWith for Option<T> {
 }
 
 pub fn run_main() -> Result<()> {
-    const POSSIBLE_LANGUAGES: &[&str] = &["kotlin", "python", "swift", "gecko_js"];
+    const POSSIBLE_LANGUAGES: &[&str] = &["kotlin", "python", "swift", "gecko_js", "ruby"];
     let matches = clap::App::new("uniffi-bindgen")
         .about("Scaffolding and bindings generator for Rust")
         .version(clap::crate_version!())
